@@ -29,15 +29,10 @@
 
 /* LSB bit of I2C address signifies write (0) or read (1) */
 
-/* I2C Address */
-//#define VV6450_ADDR		(0x10<<1)
-#define VV6450_ADDR		(0xE0)
-
 /* I2C Registers */
 
 /* Status registers */
-#define VV6450_DEVICEH		0x00	/* Chip identification number including revision indicator */
-#define VV6450_DEVICEL		0x01
+#define VV6450_DEVICEH		0xE00A	/* Chip identification number including revision indicator */
 #define VV6450_STATUS0		0x02	/* User can determine whether timed I2C data has been consumed by interrogating flag states */
 #define VV6450_LINECOUNTH	0x03	/* Current line counter value */
 #define VV6450_LINECOUNTL	0x04
@@ -114,72 +109,51 @@ static int vv6450_set_size(struct quickcam *qc, unsigned int width, unsigned int
 
 	PDEBUG("vv6450_set_size(quickcam=%p,width=%i,height=%i)",qc,width,height);
 
-#if 0
-	if ( (width == (324*2)) && (height == (248*2)) )
-	{
-		sd->width = width;
-		sd->height = height;
-		//r = qc_i2c_wait(qc);
-		STV_SET_CHECK(0x1505, 0x01);
-		if(r) goto fail;
-#if 0
-		r = qc_i2c_wait(qc);
-		STV_SET_CHECK(0x143F, 0x01);
-		if(r) goto fail;
-#endif
-	}
-	else
-	if ( (width == 640) && (height == 480) )
-	{
-		sd->width = (324*2);
-		sd->height = (248*2);
-		//r = qc_i2c_wait(qc);
-		STV_SET_CHECK(0x1505, 0x01);
-		if(r) goto fail;
-	} else
-#endif
-	if ( (width == 324) && (height == 248) )
-	{
-		sd->width = width;
-		sd->height = height;
-		//r = qc_i2c_wait(qc);
-		STV_SET_CHECK(0x1505, 0x02);
-		if(r) goto fail;
-	} else
-	if ( (width == 320) && (height == 240) )
-	{
-		sd->width = 324;
-		sd->height = 248;
-		//r = qc_i2c_wait(qc);
-		STV_SET_CHECK(0x1505, 0x02);
-		if(r) goto fail;
-	} else
-	if ( (width == 248) && (height == 162) )
-	{
-		sd->width = width;
-		sd->height = height;
-		//r = qc_i2c_wait(qc);
-		STV_SET_CHECK(0x1505, 0x07);
-		if(r) goto fail;
-	} else
-	if ( (width == 162) && (height == 124) )
-	{
-		sd->width = width;
-		sd->height = height;
-		//r = qc_i2c_wait(qc);
-		STV_SET_CHECK(0x1505, 0x0f);
-		if(r) goto fail;
-	} else
-	if ( (width == 160) && (height == 120) )
+
+	if ( (width <= 162) && (height <= 124) )
 	{
 		sd->width = 162;
 		sd->height = 124;
-		//r = qc_i2c_wait(qc);
+		PDEBUG("set %dx%d", sd->width, sd->height);
 		STV_SET_CHECK(0x1505, 0x0f);
 		if(r) goto fail;
-	}
-	else
+	} else
+	if ( (width <= 162) && (height <= 248) )
 	{
+		sd->width = 162;
+		sd->height = 248;
+		PDEBUG("set %dx%d", sd->width, sd->height);
+		STV_SET_CHECK(0x1505, 0x08);
+		if(r) goto fail;
+	} else
+	if ( (width <= 324) && (height <= 124) )
+	{
+		sd->width = 324;
+		sd->height = 124;
+		PDEBUG("set %dx%d", sd->width, sd->height);
+		STV_SET_CHECK(0x1505, 0x04);
+		if(r) goto fail;
+	} else
+	if ( (width <= 324) && (height <= 248) )
+	{
+		sd->width = 324;
+		sd->height = 248;
+		PDEBUG("set %dx%d", sd->width, sd->height);
+		STV_SET_CHECK(0x1505, 0x02);
+		if(r) goto fail;
+	} else
+#ifdef TRY_HIGHRES
+	if ( (width <= 648) && (height <= 496) )
+	{
+		// this doesn't work
+		sd->width = 648;
+		sd->height = 496;
+		STV_SET_CHECK(0x1505, 0x01);
+		if(r) goto fail;
+	} else
+#endif
+	{
+	unknown:
 		PDEBUG("Unknown resolution %dx%d, set 324x248", width, height);
 		sd->width = 324;
 		sd->height = 248;
@@ -206,7 +180,7 @@ static int vv6450_start(struct quickcam *qc)
 	if (PARANOID && !qc)  { PDEBUG("qc==NULL"); return -EINVAL; }
 
 	r = qc_i2c_wait(qc);
-	STV_SET_CHECK(0x1440, 0x01);
+	STV_SET_CHECK(STV_ISO_ENABLE, 0x01);
 	return 0;
 fail:
 	//PDEBUG("Error start");
@@ -227,7 +201,7 @@ static int vv6450_stop(struct quickcam *qc)
 	if (sd->subsample) cmd |= BIT(7)|BIT(6);		/* sub-sampled QCIF mode */
 	r = qc_i2c_wait(qc);
 	
-	STV_SET_CHECK(0x1440, 0x00);
+	STV_SET_CHECK(STV_ISO_ENABLE, 0x00);
 	return 0;
 fail:
 	//PDEBUG("Error stop");
@@ -252,7 +226,7 @@ static int vv6450_init(struct quickcam *qc)
 	int r = 0;
 	static const struct stv_init stv_init[] = {
 
-/* LOGTAG */ { NULL, 0x1440, 0x00 },	/* disable capture */
+/* LOGTAG */ { NULL, STV_ISO_ENABLE, 0x00 },	/* disable capture */
 /* LOGTAG */ { NULL, 0x1436, 0x00 },
 /* LOGTAG */ { NULL, 0x1432, 0x03 },				/* 0x00-0x1F contrast ? */
 /* LOGTAG */ { NULL, 0x143a, 0xF9 },				/* 0x00-0x0F - gain */
@@ -496,8 +470,8 @@ fail:
 /* {{{ [fold] vv6450_set_target: Set target brightness for sensor autoexposure, val=0..65535 */
 static int vv6450_set_target(struct quickcam *qc, unsigned int val)
 {
-	struct qc_sensor_data *sd = &qc->sensor_data;
-	int r;
+	//struct qc_sensor_data *sd = &qc->sensor_data;
+	int r = 0;
 
 	if (val == qc->vpic.brightness) return 0;
 	qc->vpic.brightness = val;
@@ -517,6 +491,7 @@ static int vv6450_set_target(struct quickcam *qc, unsigned int val)
 	if(r) goto fail;
 	STV_SET_CHECK(0x143F, 0x01);
 #endif
+	if(r) goto fail;
 	return r;
 fail:
 	PDEBUG("error set_target");
@@ -572,20 +547,17 @@ const struct qc_sensor qc_sensor_vv6450 = {
         adapt_gainlow:  40000,
         adapt_gainhigh: 65535,
 #else
-	//adapt_gainlow:	20,
 	adapt_gainlow:	1000,
-	//adapt_gainhigh:	20000,
-        //adapt_gainlow:  25000,
-        //adapt_gainhigh: 35000,
-        //adapt_gainlow:  20000,
-        //adapt_gainhigh: 40000,
         adapt_gainhigh: 65535,
 #endif
 	/* Information needed to access the sensor via I2C */
-	reg23: 		5,
-	i2c_addr: 	VV6450_ADDR,
+	reg23: 		-1, // not used
+	i2c_addr: 	(unsigned char)-1, // not used
 	/* Identification information used for auto-detection */
 	id_reg:		VV6450_DEVICEH,
-	id:		0xf0,
-	length_id:	1,
+	/*
+	  0x08F0 & 0x08F5 is known to be Messenger
+	*/
+	id:		0x08,
+	length_id:	2,
 };
